#!/usr/bin/env python

"""
Convert markdown to html.
Use pygments to syntax highlight any code blocks that begin with a shebang.
Wrap html and style it somewhat like docs.python.org.
"""

import sys
import re
from markdown import Markdown
from markdown import TextPreprocessor
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name, TextLexer

INLINE_STYLES = False
INCLUDE_SHEBANG = False

class CodeBlockPreprocessor(TextPreprocessor):

  pattern = re.compile( r'^(?P<codeblock> {4}#!(?P<shebang>.*)\n(?P<body>(( {4}.*)?\n)*))', re.M)

  formatter = HtmlFormatter(noclasses=INLINE_STYLES)

  def run(self, lines):

    def repl(m):

      # extract program from shebang
      argv = m.group('shebang').split()
      program = argv[0].split('/')[-1]
      if program == 'env' and len(argv) >= 2:
        program = argv[1]

      try:
        lexer = get_lexer_by_name(program)
      except ValueError:
        lexer = TextLexer()

      if INCLUDE_SHEBANG:
        code = m.group('codeblock')
      else:
        code = m.group('body')

      code = highlight(code, lexer, self.formatter)
      code = code.replace('\n\n', '\n&nbsp;\n').replace('\n', '<br />')
      return '\n\n<div class="code">%s</div>\n\n' % code

    return self.pattern.sub(repl, lines)


if len(sys.argv) > 1:
  infile = file(sys.argv[1])
else:
  infile = sys.stdin

md = Markdown()
md.textPreprocessors.insert(0, CodeBlockPreprocessor())
html = md.convert(infile.read())

template = """
<html>
<head>
  <link rel="stylesheet" href="default.css" type="text/css" />
  <link rel="stylesheet" href="pygments.css" type="text/css" />
</head>
<body>
<div class="body">
%s
</div>
</body>
</html>
"""

print template % html




#!/usr/bin/env python

"""flattery - fast flattening and unflattening of nested Python data

This library exposes a fast C implementation for flattening and unflattening hierarchical Python data structures. A unit test suite is included. See README.md for usage examples.
"""

classifiers = """\
Development Status :: 4 - Beta
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Programming Language :: Python
Topic :: Text Processing :: General
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: OS Independent
"""

from distutils.core import setup, Extension

doclines = __doc__.split("\n")

setup(
  name='flattery',
  version='0.1',
  ext_modules=[
    Extension(
      'flattery.cext',
      [ 'flattery.c' ],
      extra_compile_args=['-fPIC'],
      define_macros=[],
    ),
  ],
  packages=['flattery'],
  maintainer="Alan Grow",
  maintainer_email="alangrow+flattery@gmail.com",
  url="https://github.com/acg/python-flattery",
  license = "https://github.com/acg/python-flattery/blob/master/LICENSE",
  platforms = ["any"],
  description = doclines[0],
  classifiers = filter(None, classifiers.split("\n")),
  long_description = "\n".join(doclines[2:]),
)




#!/usr/bin/env python

import unittest
from flattery import flatten, unflatten


class FunctionTestCase(unittest.TestCase):

  def __init__(self, **keywords):
    unittest.TestCase.__init__(self)
    self.args = []
    self.kwargs = {}

    for k, v in keywords.items():
      setattr(self, k, v)

  def runTest(self):
    self.assertEqual( self.func(*self.args,**self.kwargs), self.expected )

  def shortDescription(self):
    return self.name


# TODO add more coverage
# TODO add bi-directional tests of flatten()

TEST_UNFLATTEN = 0x1
TEST_FLATTEN = 0x2
TEST_BOTH = TEST_UNFLATTEN | TEST_FLATTEN

TRUTH = [
  ("empty", TEST_BOTH, {}, {}),
  ("empty key", TEST_BOTH, {'':1}, {'':1}),
  ("depth 2 path x 1", TEST_BOTH, {"a.b":42}, {"a":{"b":42}}),
  ("depth 2 path x 2", TEST_BOTH, {"a.b":42,"c.d":"x"}, {"a":{"b":42},"c":{"d":"x"}}),
  ("depth 2 path x 2, overlap", TEST_BOTH, {"a.b":42,"a.c":"x"}, {"a":{"b":42,"c":"x"}}),
  ("simple list", TEST_BOTH, {"a.0":1,"a.1":2}, {"a":[1,2]}),
  ("sparse list", TEST_BOTH, {"a.0":1,"a.9":10}, {"a":[1,None,None,None,None,None,None,None,None,10]}),
  ("deep dict", TEST_BOTH, {"a.b.c.d.e":1}, {"a":{"b":{"c":{"d":{"e":1}}}}}),
  ("list under dict", TEST_BOTH, {"a.b.0":1,"a.b.1":2}, {"a":{"b":[1,2]}}),
  ("dict under list x 1", TEST_BOTH, {"a.0.b":1}, {"a":[{"b":1}]}),
  ("dict under list x 2, overlap", TEST_BOTH, {"a.0.b":1,"a.0.c":2}, {"a":[{"b":1,"c":2}]}),
  ("deep list", TEST_BOTH, {"a.0.0.0.0":42}, {"a":[[[[42]]]]}),
]


def main():

  suite = unittest.TestSuite()
  i = 0

  for name, mode, flat, unflat in TRUTH:

    if mode & TEST_FLATTEN:

      suite.addTest(FunctionTestCase(
        name="test %d flatten %s" % (i,name),
        func=flatten,
        args=[unflat],
        expected=flat,
      ))
      i += 1

    if mode & TEST_UNFLATTEN:

      suite.addTest(FunctionTestCase(
        name="test %d unflatten %s" % (i,name),
        func=unflatten,
        args=[flat],
        expected=unflat,
      ))
      i += 1

  runner = unittest.TextTestRunner(verbosity=2)
  results = runner.run(suite)

  if len(results.failures) or len(results.errors):
    return 1
  else:
    return 0


if __name__ == '__main__':
  import sys
  sys.exit(main())

